Package: PhenotypeR
Type: Package
Title: Assess Study Cohorts Using a Common Data Model
Version: 0.5.0
Authors@R: c(
    person("Edward", "Burn", , "edward.burn@ndorms.ox.ac.uk", 
    role = c("aut", "cre"), comment = c(ORCID = "0000-0002-9286-1128")),
    person("Martí", "Català", , "marti.catalasabate@ndorms.ox.ac.uk", 
    role = c("aut"), comment = c(ORCID = "0000-0003-3308-9905")),
    person("Xihang", "Chen", , "xihang.chen@ndorms.ox.ac.uk", 
    role = c("aut"), comment = c(ORCID = "0009-0001-8112-8959")),
    person("Marta", "Alcalde-Herraiz", , "marta.alcaldeherraiz@ndorms.ox.ac.uk", 
    role = c("aut"), comment = c(ORCID = "0009-0002-4405-1814")),
    person("Nuria", "Mercade-Besora", , "nuria.mercadebesora@ndorms.ox.ac.uk", 
    role = c("aut"), comment = c(ORCID = "0009-0006-7948-3747")),
    person("Albert", "Prats-Uribe", , "albert.prats-uribe@ndorms.ox.ac.uk", 
    role = "aut", comment = c(ORCID = "0000-0003-1202-9153")))
Description: Phenotype study cohorts in data mapped to the 
    Observational Medical Outcomes Partnership Common Data Model. Diagnostics 
    are run at the database, code list, cohort, and population level to assess 
    whether study cohorts are ready for research.
License: Apache License (>= 2)
Encoding: UTF-8
Depends: R (>= 4.1.0)
Suggests: CDMConnector (>= 1.6.1), duckdb, DBI, gt, omock, testthat (>=
        3.0.0), knitr, glue, RPostgres, ggplot2, stringr, shiny (>=
        1.11.1), DiagrammeR, DiagrammeRsvg, reactable, rsvg, sortable,
        shinycssloaders, here, DT, bslib, shinyWidgets, plotly, tidyr,
        scales, usethis, rmarkdown, CohortSurvival (>= 1.1.0), ellmer,
        htmltools, visOmopResults (>= 1.4.2), rsconnect, cpp11,
        progress, qs2, lubridate, systemfonts, officer, fs,
        OmopConstructor, tools, jsonlite, jsonvalidate, shinyjs
Config/testthat/edition: 3
RoxygenNote: 7.3.3
Imports: cli, clock, CodelistGenerator (>= 4.0.2),
        CohortCharacteristics (>= 1.1.3), CohortConstructor (>= 0.6.2),
        dplyr, DrugUtilisation (>= 1.1.0), IncidencePrevalence (>=
        1.2.0), MeasurementDiagnostics (>= 0.3.0), omopgenerics (>=
        1.2.0), OmopSketch (>= 1.0.1), PatientProfiles (>= 1.4.5),
        purrr, readr, rlang, vctrs
URL: https://ohdsi.github.io/PhenotypeR/
BugReports: https://github.com/OHDSI/PhenotypeR/issues
VignetteBuilder: knitr
Config/testthat/parallel: true
NeedsCompilation: no
Packaged: 2026-05-26 20:58:26 UTC; orms0426
Author: Edward Burn [aut, cre] (ORCID: <https://orcid.org/0000-0002-9286-1128>),
  Martí Català [aut] (ORCID: <https://orcid.org/0000-0003-3308-9905>),
  Xihang Chen [aut] (ORCID: <https://orcid.org/0009-0001-8112-8959>),
  Marta Alcalde-Herraiz [aut] (ORCID:
    <https://orcid.org/0009-0002-4405-1814>),
  Nuria Mercade-Besora [aut] (ORCID:
    <https://orcid.org/0009-0006-7948-3747>),
  Albert Prats-Uribe [aut] (ORCID:
    <https://orcid.org/0000-0003-1202-9153>)
Maintainer: Edward Burn <edward.burn@ndorms.ox.ac.uk>
Repository: CRAN
Date/Publication: 2026-05-27 06:30:02 UTC
Built: R 4.5.2; ; 2026-05-27 06:50:02 UTC; unix
