#' @importFrom omopgenerics suppress
#' @export
omopgenerics::suppress

#' @importFrom omopgenerics settings
#' @export
omopgenerics::settings

#' @importFrom omopgenerics bind
#' @export
omopgenerics::bind

#' @importFrom omopgenerics importSummarisedResult
#' @export
omopgenerics::importSummarisedResult

#' @importFrom omopgenerics exportSummarisedResult
#' @export
omopgenerics::exportSummarisedResult

#' @importFrom CodelistGenerator summariseAchillesCodeUse
#' @export
CodelistGenerator::summariseAchillesCodeUse

#' @importFrom CodelistGenerator summariseCohortCodeUse
#' @export
CodelistGenerator::summariseCohortCodeUse

#' @importFrom CodelistGenerator summariseCodeUse
#' @export
CodelistGenerator::summariseCodeUse

#' @importFrom CodelistGenerator summariseOrphanCodes
#' @export
CodelistGenerator::summariseOrphanCodes
