% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/getClinicalDescription.R
\name{getClinicalDescription}
\alias{getClinicalDescription}
\title{Get clinical descriptions using an LLM}
\usage{
getClinicalDescription(chat, name, outputDir)
}
\arguments{
\item{chat}{An ellmer chat}

\item{name}{Clinical event of interest}

\item{outputDir}{Folder to save clinical descriptions.}
}
\value{
Creates a word document with a clinical description for each event.
}
\description{
Get clinical descriptions using an LLM
}
